@@ -160,7 +160,6 @@ MIDDLEWARE = [
     'django.middleware.clickjacking.XFrameOptionsMiddleware',
     'core.middleware.CommonMiddlewareAppendSlashWithoutRedirect',  # instead of 'CommonMiddleware'
     'core.middleware.CommonMiddleware',
-    'core.middleware.DRFResponseFormatter',
     'django_user_agents.middleware.UserAgentMiddleware',
     'core.middleware.SetSessionUIDMiddleware',
     'core.middleware.ContextLogMiddleware',
@@ -255,7 +254,9 @@ SWAGGER_SETTINGS = {
     'SECURITY_DEFINITIONS': {
         'token': {
             'type': 'token',
-            'url': '/business/account/#/token'
+            'name': 'Token',
+            'in': 'header',
+            'url': '/user/account'
         }
     },
     'APIS_SORTER': 'alpha',
@@ -264,7 +265,13 @@ SWAGGER_SETTINGS = {
     'OPERATIONS_SORTER': 'alpha'
 }
 
-SENTRY_FE = None
+SENTRY_DSN = get_env('SENTRY_DSN', None)
+SENTRY_RATE = float(get_env('SENTRY_RATE', 0.25))
+SENTRY_ENVIRONMENT = get_env('SENTRY_ENVIRONMENT', 'stage.opensource')
+SENTRY_REDIS_ENABLED = False
+FRONTEND_SENTRY_DSN = get_env('FRONTEND_SENTRY_DSN', None)
+FRONTEND_SENTRY_RATE = get_env('FRONTEND_SENTRY_RATE', 0.1)
+FRONTEND_SENTRY_ENVIRONMENT = get_env('FRONTEND_SENTRY_ENVIRONMENT', 'stage.opensource')
 
 ROOT_URLCONF = 'core.urls'
 WSGI_APPLICATION = 'core.wsgi.application'
@@ -310,17 +317,19 @@ EXPORT_URL_ROOT = '/export/'
 os.makedirs(EXPORT_DIR, exist_ok=True)
 
 # file / task size limits
-DATA_UPLOAD_MAX_MEMORY_SIZE = int(get_env('DATA_UPLOAD_MAX_MEMORY_SIZE', 50 * 1024 * 1024))
-TASKS_MAX_NUMBER = 250000
+DATA_UPLOAD_MAX_MEMORY_SIZE = int(get_env('DATA_UPLOAD_MAX_MEMORY_SIZE', 250 * 1024 * 1024))
+TASKS_MAX_NUMBER = 1000000
 TASKS_MAX_FILE_SIZE = DATA_UPLOAD_MAX_MEMORY_SIZE
 
 TASK_LOCK_TTL = int(get_env('TASK_LOCK_TTL')) if get_env('TASK_LOCK_TTL') else None
 TASK_LOCK_DEFAULT_TTL = int(get_env('TASK_LOCK_DEFAULT_TTL', 3600))
+TASK_LOCK_MIN_TTL = int(get_env('TASK_LOCK_MIN_TTL', 120))
 
 # Email backend
-FROM_EMAIL = 'Label Studio <hello@labelstud.io>'
-EMAIL_BACKEND = 'django.core.mail.backends.dummy.EmailBackend'
+FROM_EMAIL = get_env('FROM_EMAIL', 'Label Studio <hello@labelstud.io>')
+EMAIL_BACKEND = get_env('EMAIL_BACKEND', 'django.core.mail.backends.dummy.EmailBackend')
 
+ENABLE_LOCAL_FILES_STORAGE = get_bool_env('ENABLE_LOCAL_FILES_STORAGE', default=True)
 LOCAL_FILES_SERVING_ENABLED = get_bool_env('LOCAL_FILES_SERVING_ENABLED', default=False)
 
 """ React Libraries: do not forget to change this dir in /etc/nginx/nginx.conf """
@@ -342,18 +351,10 @@ DATA_UNDEFINED_NAME = '$undefined$'
 LICENSE = {}
 VERSIONS = {}
 VERSION_EDITION = 'Community Edition'
+LATEST_VERSION_CHECK = True
+VERSIONS_CHECK_TIME = 0
+ALLOW_ORGANIZATION_WEBHOOKS = get_bool_env('ALLOW_ORGANIZATION_WEBHOOKS', False)
 
-
-def project_delete(project):
-    project.delete()
-
-
-def user_auth(user_model, email, password):
-    return None
-
-
-PROJECT_DELETE = project_delete
-USER_AUTH = user_auth
 CREATE_ORGANIZATION = 'organizations.functions.create_organization'
 GET_OBJECT_WITH_CHECK_AND_LOG = 'core.utils.get_object.get_object_with_check_and_log'
 SAVE_USER = 'users.functions.save_user'
@@ -366,6 +367,24 @@ TASK_MIXIN = 'core.mixins.DummyModelMixin'
 ANNOTATION_MIXIN = 'core.mixins.DummyModelMixin'
 ORGANIZATION_MIXIN = 'core.mixins.DummyModelMixin'
 USER_MIXIN = 'users.mixins.UserMixin'
+GET_STORAGE_LIST = 'io_storages.functions.get_storage_list'
+
+
+def project_delete(project):
+    project.delete()
+
+
+def user_auth(user_model, email, password):
+    return None
+
+
+def collect_versions_dummy(**kwargs):
+    return {}
+
+
+PROJECT_DELETE = project_delete
+USER_AUTH = user_auth
+COLLECT_VERSIONS = collect_versions_dummy
 
 # fix a problem with Windows mimetypes for JS and PNG
 import mimetypes
