@@ -3,7 +3,8 @@
 import shutil
 import io
 import hashlib
-import json
+import logging
+import ujson as json
 import os
 from datetime import datetime
 from copy import deepcopy
@@ -18,6 +19,9 @@ from tasks.models import Annotation
 # Create your models here.
 
 
+logger = logging.getLogger(__name__)
+
+
 class DataExport(object):
 
     @staticmethod
@@ -27,6 +31,11 @@ class DataExport(object):
         filename_results = os.path.join(settings.EXPORT_DIR, name + '.json')
         filename_info = os.path.join(settings.EXPORT_DIR, name + '-info.json')
         annotation_number = Annotation.objects.filter(task__project=project).count()
+        try:
+            platform_version = version.get_git_version()
+        except:
+            platform_version = 'none'
+            logger.error('Version is not detected in save_export_files()')
         info = {
             'project': {
                 'title': project.title,
@@ -37,7 +46,7 @@ class DataExport(object):
                 'annotation_number': annotation_number
             },
             'platform': {
-                'version': version.get_git_version()
+                'version': platform_version
             },
             'download': {
                 'GET': dict(get_args),
