@@ -6,16 +6,17 @@ from rest_framework import serializers
 from tasks.models import Task, Annotation
 from tasks.serializers import PredictionSerializer
 from users.models import User
+from core.label_config import replace_task_data_undefined_with_config_field
 
 
-class UserSerializer(serializers.ModelSerializer):
+class CompletedBySerializer(serializers.ModelSerializer):
     class Meta:
         model = User
         fields = ['id', 'email', 'first_name', 'last_name']
 
 
 class AnnotationSerializer(serializers.ModelSerializer):
-    completed_by = UserSerializer(read_only=True)
+    completed_by = CompletedBySerializer(read_only=True)
 
     class Meta:
         model = Annotation
@@ -31,12 +32,8 @@ class ExportDataSerializer(serializers.ModelSerializer):
     def to_representation(self, task):
         project = task.project
         data = task.data
-        data_types_keys = project.data_types.keys()
-
-        if settings.DATA_UNDEFINED_NAME in data and data_types_keys:
-            key = list(data_types_keys)[0]
-            data[key] = data[settings.DATA_UNDEFINED_NAME]
-            del data[settings.DATA_UNDEFINED_NAME]
+        
+        replace_task_data_undefined_with_config_field(data, project)
 
         return super().to_representation(task)
 
