@@ -8,6 +8,7 @@ import json
 from django.conf import settings
 from django.db import transaction
 from drf_yasg.utils import swagger_auto_schema
+from django.utils.decorators import method_decorator
 from rest_framework import generics, status
 from rest_framework.response import Response
 from rest_framework.parsers import FormParser, JSONParser, MultiPartParser
@@ -86,60 +87,65 @@ task_create_response_scheme = {
 }
 
 
+@method_decorator(name='post', decorator=swagger_auto_schema(
+        tags=['Import'],
+        responses=task_create_response_scheme,
+        operation_summary='Import tasks',
+        operation_description="""
+            Import data as labeling tasks in bulk using this API endpoint. You can use this API endpoint to import multiple tasks. 
+            One POST request is limited at 250K tasks and 200 MB.
+            
+            **Note:** Imported data is verified against a project *label_config* and must
+            include all variables that were used in the *label_config*. For example,
+            if the label configuration has a *$text* variable, then each item in a data object
+            must include a "text" field.
+            <br>
+            
+            ## POST requests
+            <hr style="opacity:0.3">
+            
+            There are three possible ways to import tasks with this endpoint:
+            
+            ### 1\. **POST with data**
+            Send JSON tasks as POST data. Only JSON is supported for POSTing files directly.
+            Update this example to specify your authorization token and Label Studio instance host, then run the following from
+            the command line.
+
+            ```bash
+            curl -H 'Content-Type: application/json' -H 'Authorization: Token abc123' \\
+            -X POST '{host}/api/projects/1/import' --data '[{{"text": "Some text 1"}}, {{"text": "Some text 2"}}]'
+            ```
+            
+            ### 2\. **POST with files**
+            Send tasks as files. You can attach multiple files with different names.
+            
+            - **JSON**: text files in JavaScript object notation format
+            - **CSV**: text files with tables in Comma Separated Values format
+            - **TSV**: text files with tables in Tab Separated Value format
+            - **TXT**: simple text files are similar to CSV with one column and no header, supported for projects with one source only
+            
+            Update this example to specify your authorization token, Label Studio instance host, and file name and path,
+            then run the following from the command line:
+
+            ```bash
+            curl -H 'Authorization: Token abc123' \\
+            -X POST '{host}/api/projects/1/import' -F ‘file=@path/to/my_file.csv’
+            ```
+            
+            ### 3\. **POST with URL**
+            You can also provide a URL to a file with labeling tasks. Supported file formats are the same as in option 2.
+            
+            ```bash
+            curl -H 'Authorization: Token abc123' \\
+            -X POST '{host}/api/projects/1/import' \\
+            --data '[{{"url": "http://example.com/test1.csv"}}, {{"url": "http://example.com/test2.csv"}}]'
+            ```
+            
+            <br>
+        """.format(host=(settings.HOSTNAME or 'https://localhost:8080'))
+    ))
 # Import
 class ImportAPI(generics.CreateAPIView):
-    """post:
-    Import tasks
-
-    Importing data as labeling tasks in bulk using this API endpoint works the same as using the Import button on the 
-    Data Manager page. You can use this API endpoint for importing multiple tasks. One POST request is limited at 250K tasks and 200 MB.
-
-    **Note:** Imported data is verified against a project *label_config* and must
-    include all variables that were used in the *label_config*. For example,
-    if the label configuration has a *$text* variable, then each item in a data object
-    must include a "text" field.
-
-
-    <br>
-
-    ## POST requests
-
-    <hr style="opacity:0.3">
-
-    There are three possible ways to import tasks with this endpoint:
-
-    1\. **POST with data**: Send JSON tasks as POST data. Only JSON is supported for POSTing files directly.
-
-    ```bash
-    curl -H 'Content-Type: application/json' -H 'Authorization: Token abc123' \\
-    -X POST 'http://localhost/api/projects/1/import' --data '[{"text": "Some text 1"}, {"text": "Some text 2"}]'
-    ```
-
-     2\. **POST with files**: Send tasks as files. You can attach multiple files with different names. 
-
-
-    - **JSON**: text files in javascript object notation format
-    - **CSV**: text files with tables in Comma Separated Values format
-    - **TSV**: text files with tables in Tab Separated Value format
-    - **TXT**: simple text files are similar to CSV with one column and no header, supported for projects with one source only
-
-    ```bash
-    curl -H 'Content-Type: application/json' -H 'Authorization: Token abc123' \\
-    -X POST 'http://localhost/api/projects/1/import' -F ‘file=@path/to/my_file.csv’
-    ```
-
-    3\. **POST with URL**: You can also provide a URL to a file with labeling tasks. Supported file formats are the same as in option 2.
-
-    ```bash
-    curl -H 'Content-Type: application/json' -H 'Authorization: Token abc123' \\
-    -X POST 'http://localhost/api/projects/1/import' \\
-    --data '[{"url": "http://example.com/test1.csv"}, {"url": "http://example.com/test2.csv"}]'
-    ```
-
-    <br>
-
-    """
-
     permission_required = all_permissions.projects_change
     parser_classes = (JSONParser, MultiPartParser, FormParser)
     serializer_class = ImportApiSerializer
@@ -153,7 +159,6 @@ class ImportAPI(generics.CreateAPIView):
             project = None
         return {'project': project, 'user': self.request.user}
 
-    @swagger_auto_schema(tags=['Import'], responses=task_create_response_scheme)
     def post(self, *args, **kwargs):
         return super(ImportAPI, self).post(*args, **kwargs)
 
@@ -217,21 +222,28 @@ class TasksBulkCreateAPI(ImportAPI):
 
 
 class ReImportAPI(ImportAPI):
-    """post:
-    Re-import tasks
-
-    Re-import tasks using the specified file upload IDs for a specific project.
-    """
     permission_required = all_permissions.projects_change
 
     @retry_database_locked()
     def create(self, request, *args, **kwargs):
         start = time.time()
         files_as_tasks_list = bool_from_request(request.data, 'files_as_tasks_list', True)
+        file_upload_ids = self.request.data.get('file_upload_ids')
 
         # check project permissions
         project = generics.get_object_or_404(Project.objects.for_user(self.request.user), pk=self.kwargs['pk'])
-        file_upload_ids = self.request.data.get('file_upload_ids')
+        
+        if not file_upload_ids:
+            return Response({
+                'task_count': 0,
+                'annotation_count': 0,
+                'prediction_count': 0,
+                'duration': 0,
+                'file_upload_ids': [],
+                'found_formats': {},
+                'data_columns': []
+            }, status=status.HTTP_204_NO_CONTENT)
+
         tasks, found_formats, data_columns = FileUpload.load_tasks_from_uploaded_files(
             project, file_upload_ids,  files_as_tasks_list=files_as_tasks_list)
 
@@ -264,26 +276,34 @@ class ReImportAPI(ImportAPI):
             'data_columns': data_columns
         }, status=status.HTTP_201_CREATED)
 
-    @swagger_auto_schema(auto_schema=None)
+    @swagger_auto_schema(
+        auto_schema=None,
+        operation_summary='Re-import tasks',
+        operation_description="""
+        Re-import tasks using the specified file upload IDs for a specific project.
+        """
+    )
     def post(self, *args, **kwargs):
         return super(ReImportAPI, self).post(*args, **kwargs)
 
 
+@method_decorator(name='get', decorator=swagger_auto_schema(
+        tags=['Import'],
+        operation_summary='Get files list',
+        operation_description="""
+        Retrieve the list of uploaded files used to create labeling tasks for a specific project.
+        """
+        ))
+@method_decorator(name='delete', decorator=swagger_auto_schema(
+        tags=['Import'],
+        operation_summary='Delete files',
+        operation_description="""
+        Delete uploaded files for a specific project.
+        """
+        ))
 class FileUploadListAPI(generics.mixins.ListModelMixin,
                         generics.mixins.DestroyModelMixin,
                         generics.GenericAPIView):
-    """
-    get:
-    Get files list
-
-    Retrieve the list of uploaded files used to create labeling tasks for a specific project.
-
-    delete:
-    Delete files
-
-    Delete uploaded files for a specific project.
-    """
-
     parser_classes = (JSONParser, MultiPartParser, FormParser)
     serializer_class = FileUploadSerializer
     permission_required = ViewClassPermission(
@@ -304,11 +324,9 @@ class FileUploadListAPI(generics.mixins.ListModelMixin,
         logger.debug(f'File Upload IDs found: {ids}')
         return FileUpload.objects.filter(project_id=project.id, id__in=ids, user=self.request.user)
 
-    @swagger_auto_schema(tags=['Import'])
     def get(self, request, *args, **kwargs):
         return self.list(request, *args, **kwargs)
 
-    @swagger_auto_schema(tags=['Import'])
     def delete(self, request, *args, **kwargs):
         project = generics.get_object_or_404(Project.objects.for_user(self.request.user),  pk=self.kwargs['pk'])
         ids = self.request.data.get('file_upload_ids')
@@ -320,22 +338,33 @@ class FileUploadListAPI(generics.mixins.ListModelMixin,
             raise ValueError('"file_upload_ids" parameter must be a list of integers')
         return Response({'deleted': deleted}, status=status.HTTP_200_OK)
 
-
+@method_decorator(name='get', decorator=swagger_auto_schema(
+        tags=['Import'],
+        operation_summary='Get file upload',
+        operation_description='Retrieve details about a specific uploaded file.'
+    ))
+@method_decorator(name='patch', decorator=swagger_auto_schema(
+        tags=['Import'],
+        operation_summary='Update file upload',
+        operation_description='Update a specific uploaded file.',
+        request_body=FileUploadSerializer
+    ))
+@method_decorator(name='delete', decorator=swagger_auto_schema(
+        tags=['Import'],
+        operation_summary='Delete file upload',
+        operation_description='Delete a specific uploaded file.'))
 class FileUploadAPI(generics.RetrieveUpdateDestroyAPIView):
     parser_classes = (JSONParser, MultiPartParser, FormParser)
     permission_classes = (IsAuthenticated, )
     serializer_class = FileUploadSerializer
     queryset = FileUpload.objects.all()
 
-    @swagger_auto_schema(tags=['Import'], operation_summary='Get file upload', operation_description='Retrieve details about a specific uploaded file.')
     def get(self, *args, **kwargs):
         return super(FileUploadAPI, self).get(*args, **kwargs)
 
-    @swagger_auto_schema(tags=['Import'], operation_summary='Update file upload', operation_description='Update a specific uploaded file.', request_body=FileUploadSerializer)
     def patch(self, *args, **kwargs):
         return super(FileUploadAPI, self).patch(*args, **kwargs)
 
-    @swagger_auto_schema(tags=['Import'], operation_summary='Delete file upload', operation_description='Delete a specific uploaded file.')
     def delete(self, *args, **kwargs):
         return super(FileUploadAPI, self).delete(*args, **kwargs)
 
