@@ -29,7 +29,10 @@ class FileUpload(models.Model):
 
     @property
     def url(self):
-        return self.file.url
+        if settings.HOSTNAME:
+            return settings.HOSTNAME + self.file.url
+        else:
+            return self.file.url
 
     @property
     def format(self):
@@ -44,7 +47,7 @@ class FileUpload(models.Model):
 
     @property
     def content(self):
-        with io.open(self.filepath) as f:
+        with io.open(self.filepath, encoding='utf-8') as f:
             return f.read()
 
     def read_tasks_list_from_csv(self, sep=','):
