@@ -12,7 +12,6 @@ from data_manager.models import View
 from tasks.models import Task
 
 
-DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
 TASKS = 'tasks:'
 logger = logging.getLogger(__name__)
 
@@ -125,7 +124,7 @@ def get_all_columns(project):
         },
         {
             'id': 'annotations_results',
-            'title': "Annotations results",
+            'title': "Annotation results",
             'type': "String",
             'target': 'tasks',
             'help': 'Annotation results stacked over all annotations',
@@ -136,7 +135,7 @@ def get_all_columns(project):
         },
         {
             'id': 'predictions_score',
-            'title': "Predictions score",
+            'title': "Prediction score",
             'type': "Number",
             'target': 'tasks',
             'help': 'Average prediction score over all task predictions',
@@ -147,7 +146,7 @@ def get_all_columns(project):
         },
         {
             'id': 'predictions_results',
-            'title': "Predictions results",
+            'title': "Prediction results",
             'type': "String",
             'target': 'tasks',
             'help': 'Prediction results stacked over all predictions',
@@ -184,6 +183,7 @@ def get_all_columns(project):
             'type': 'List',
             'target': 'tasks',
             'help': 'All users who completed the task',
+            'schema': { 'items': project.organization.members.values_list('user__id', flat=True) },
             'visibility_defaults': {
                 'explore': True,
                 'labeling': False
@@ -213,7 +213,7 @@ def get_prepared_queryset(request, project):
                                        '"excluded | included": [...task_ids...]}')
         filters = request.data.get('filters', None)
         ordering = request.data.get('ordering', [])
-        prepare_params = PrepareParams(project=project.id, selectedItems=selected,
+        prepare_params = PrepareParams(project=project.id, selectedItems=selected, data=request.data,
                                        filters=filters, ordering=ordering)
 
     queryset = Task.prepared.all(prepare_params=prepare_params)
