@@ -6,15 +6,15 @@ from pydantic import BaseModel
 
 
 class FilterIn(BaseModel):
-    min: Union[int, float, str]
-    max: Union[int, float, str]
+    min: Union[float, int, str]
+    max: Union[float, int, str]
 
 
 class Filter(BaseModel):
     filter: str
     operator: str
     type: str
-    value: Union[str, int, float, bool, FilterIn]
+    value: Union[str, float, int, bool, FilterIn]
 
 
 class ConjunctionEnum(Enum):
@@ -38,3 +38,4 @@ class PrepareParams(BaseModel):
     ordering: List[str] = []
     selectedItems: Optional[SelectedItems] = None
     filters: Optional[Filters] = None
+    data: Optional[dict] = None
