@@ -7,7 +7,7 @@ from django.db import transaction
 
 from data_manager.models import View, Filter, FilterGroup
 from tasks.models import Task
-from tasks.serializers import TaskSerializer, AnnotationSerializer, PredictionSerializer
+from tasks.serializers import TaskSerializer, AnnotationSerializer, PredictionSerializer, AnnotationDraftSerializer
 
 
 class FilterSerializer(serializers.ModelSerializer):
@@ -158,6 +158,7 @@ class ViewSerializer(serializers.ModelSerializer):
 class DataManagerTaskSerializer(TaskSerializer):
     predictions = PredictionSerializer(many=True, default=[], read_only=True)
     annotations = AnnotationSerializer(many=True, default=[], read_only=True)
+    drafts = serializers.SerializerMethodField()
 
     cancelled_annotations = serializers.SerializerMethodField()
     completed_at = serializers.SerializerMethodField()
@@ -186,6 +187,7 @@ class DataManagerTaskSerializer(TaskSerializer):
             "total_predictions",
             "annotations",
             "predictions",
+            "drafts",
             "file_upload",
             "annotators",
             "project"
@@ -198,7 +200,7 @@ class DataManagerTaskSerializer(TaskSerializer):
     @staticmethod
     def get_completed_at(obj):
         annotations = obj.annotations.all()
-        if annotations:
+        if obj.is_labeled and annotations:
             return max(c.created_at for c in annotations)
         return None
 
@@ -241,6 +243,19 @@ class DataManagerTaskSerializer(TaskSerializer):
         result = [r for r in result if r is not None]
         return result
 
+    def get_drafts(self, task):
+        """Return drafts only for the current user"""
+        # it's for swagger documentation
+        if not isinstance(task, Task):
+            return AnnotationDraftSerializer(many=True)
+
+        drafts = task.drafts
+        if 'request' in self.context and hasattr(self.context['request'], 'user'):
+            user = self.context['request'].user
+            drafts = drafts.filter(user=user)
+
+        return AnnotationDraftSerializer(drafts, many=True, read_only=True, default=True, context=self.context).data
+
 
 class SelectedItemsSerializer(serializers.Serializer):
     all = serializers.BooleanField()
