@@ -3,16 +3,18 @@
 import logging
 
 from django.urls import reverse
+from django.conf import settings
 from rest_framework.parsers import FormParser, JSONParser, MultiPartParser
 from rest_framework import generics
 from rest_framework.views import APIView
 from rest_framework.response import Response
 
 from drf_yasg.utils import swagger_auto_schema
+from django.utils.decorators import method_decorator
 
-from core.mixins import APIViewVirtualRedirectMixin, APIViewVirtualMethodMixin
-from core.permissions import all_permissions, ViewClassPermission
-from core.utils.common import get_object_with_check_and_log
+from label_studio.core.mixins import APIViewVirtualRedirectMixin, APIViewVirtualMethodMixin
+from label_studio.core.permissions import all_permissions, ViewClassPermission
+from label_studio.core.utils.common import get_object_with_check_and_log
 
 from organizations.models import Organization
 from organizations.serializers import (
@@ -23,15 +25,23 @@ from organizations.serializers import (
 logger = logging.getLogger(__name__)
 
 
+@method_decorator(name='get', decorator=swagger_auto_schema(
+        tags=['Organizations'],
+        operation_summary='List your organizations',
+        operation_description="""
+        Return a list of the organizations you've created or that you have access to.
+        """
+    ))
 class OrganizationListAPI(generics.ListCreateAPIView):
-    """
-    get:
-    List your organizations
-
-    Return a list of the organizations you've created.
-    """
+    queryset = Organization.objects.all()
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    permission_required = all_permissions.organizations_change
+    permission_required = ViewClassPermission(
+        GET=all_permissions.organizations_view,
+        PUT=all_permissions.organizations_change,
+        POST=all_permissions.organizations_create,
+        PATCH=all_permissions.organizations_change,
+        DELETE=all_permissions.organizations_change,
+    )
     serializer_class = OrganizationIdSerializer
 
     def get_object(self):
@@ -39,10 +49,9 @@ class OrganizationListAPI(generics.ListCreateAPIView):
         self.check_object_permissions(self.request, org)
         return org
 
-    def get_queryset(self):
-        return Organization.objects.filter(created_by=self.request.user)
+    def filter_queryset(self, queryset):
+        return queryset.filter(users=self.request.user).distinct()
 
-    @swagger_auto_schema(tags=['Organizations'])
     def get(self, request, *args, **kwargs):
         return super(OrganizationListAPI, self).get(request, *args, **kwargs)
 
@@ -51,13 +60,12 @@ class OrganizationListAPI(generics.ListCreateAPIView):
         return super(OrganizationListAPI, self).post(request, *args, **kwargs)
 
 
+@method_decorator(name='get', decorator=swagger_auto_schema(
+        tags=['Organizations'],
+        operation_summary='Get organization members list',
+        operation_description='Retrieve a list of the organization members and their IDs.'
+    ))
 class OrganizationMemberListAPI(generics.ListAPIView):
-    """
-    get:
-    Get organization members list
-
-    Retrieve a list of the organization members.
-    """
 
     parser_classes = (JSONParser, FormParser, MultiPartParser)
     permission_required = ViewClassPermission(
@@ -70,27 +78,26 @@ class OrganizationMemberListAPI(generics.ListAPIView):
 
     def get_queryset(self):
         org = generics.get_object_or_404(self.request.user.organizations, pk=self.kwargs[self.lookup_field])
-        return org.members
+        return org.members.order_by('user__username')
 
-    @swagger_auto_schema(tags=['Organizations'])
     def get(self, request, *args, **kwargs):
         return super(OrganizationMemberListAPI, self).get(request, *args, **kwargs)
 
 
+@method_decorator(name='get', decorator=swagger_auto_schema(
+        tags=['Organizations'],
+        operation_summary=' Get organization settings',
+        operation_description='Retrieve the settings for a specific organization by ID.'
+    ))
+@method_decorator(name='patch', decorator=swagger_auto_schema(
+        tags=['Organizations'],
+        operation_summary='Update organization settings',
+        operation_description='Update the settings for a specific organization by ID.'
+    ))
 class OrganizationAPI(APIViewVirtualRedirectMixin,
                       APIViewVirtualMethodMixin,
                       generics.RetrieveUpdateAPIView):
-    """
-    get:
-    Get organization settings
 
-    Retrieve the settings for a specific organization.
-
-    patch:
-    Update organization settings
-
-    Update the settings for a specific organization.
-    """
     parser_classes = (JSONParser, FormParser, MultiPartParser)
     queryset = Organization.objects.all()
     permission_required = all_permissions.organizations_change
@@ -104,11 +111,9 @@ class OrganizationAPI(APIViewVirtualRedirectMixin,
         self.check_object_permissions(self.request, org)
         return org
 
-    @swagger_auto_schema(tags=['Organizations'])
     def get(self, request, *args, **kwargs):
         return super(OrganizationAPI, self).get(request, *args, **kwargs)
 
-    @swagger_auto_schema(tags=['Organizations'])
     def patch(self, request, *args, **kwargs):
         return super(OrganizationAPI, self).patch(request, *args, **kwargs)
 
@@ -121,33 +126,37 @@ class OrganizationAPI(APIViewVirtualRedirectMixin,
         return super(OrganizationAPI, self).put(request, *args, **kwargs)
 
 
+@method_decorator(name='get', decorator=swagger_auto_schema(
+        tags=["Invites"],
+        operation_summary='Get organization invite link',
+        operation_description='Get a link to use to invite a new member to an organization in Label Studio Enterprise.',
+        responses={200: OrganizationInviteSerializer()}
+    ))
 class OrganizationInviteAPI(APIView):
     parser_classes = (JSONParser,)
     permission_required = all_permissions.organizations_change
 
-    @swagger_auto_schema(
-        tags=["Invites"],
-        operation_summary='Get organization invite link',
-        responses={200: OrganizationInviteSerializer()}
-    )
     def get(self, request, *args, **kwargs):
         org = get_object_with_check_and_log(self.request, Organization, pk=request.user.active_organization_id)
         self.check_object_permissions(self.request, org)
         invite_url = '{}?token={}'.format(reverse('user-signup'), org.token)
+        if hasattr(settings, 'FORCE_SCRIPT_NAME') and settings.FORCE_SCRIPT_NAME:
+            invite_url = invite_url.replace(settings.FORCE_SCRIPT_NAME, '', 1)
         serializer = OrganizationInviteSerializer(data={'invite_url': invite_url, 'token': org.token})
         serializer.is_valid()
         return Response(serializer.data, status=200)
 
 
+@method_decorator(name='post', decorator=swagger_auto_schema(
+        tags=["Invites"],
+        operation_summary='Reset organization token',
+        operation_description='Reset the token used in the invitation link to invite someone to an organization.',
+        responses={200: OrganizationInviteSerializer()}
+    ))
 class OrganizationResetTokenAPI(APIView):
     permission_required = all_permissions.organizations_invite
     parser_classes = (JSONParser,)
 
-    @swagger_auto_schema(
-        tags=["Invites"],
-        operation_summary='Reset organization token',
-        responses={200: OrganizationInviteSerializer()}
-    )
     def post(self, request, *args, **kwargs):
         org = request.user.active_organization
         org.reset_token()
@@ -156,4 +165,3 @@ class OrganizationResetTokenAPI(APIView):
         serializer = OrganizationInviteSerializer(data={'invite_url': invite_url, 'token': org.token})
         serializer.is_valid()
         return Response(serializer.data, status=201)
-
