@@ -11,10 +11,9 @@ from django.core.validators import MinLengthValidator, MaxLengthValidator
 from django.db import transaction, models
 from annoying.fields import AutoOneToOneField
 
-from rest_framework.exceptions import ValidationError
-
-from tasks.models import Task, Prediction, Annotation, Q_task_finished_annotations, Q_finished_annotations
-from core.utils.common import create_hash, pretty_date, sample_query, get_attr_or_item, load_func
+from tasks.models import Task, Prediction, Annotation, Q_task_finished_annotations, bulk_update_stats_project_tasks
+from core.utils.common import create_hash, sample_query, get_attr_or_item, load_func
+from core.utils.exceptions import LabelStudioValidationErrorSentryIgnored
 from core.label_config import (
     parse_config, validate_label_config, extract_data_types, get_all_object_tag_names, config_line_stipped,
     get_sample_task, get_all_labels, get_all_control_tag_tuples, get_annotation_tuple
@@ -35,6 +34,13 @@ class ProjectManager(models.Manager):
                 'tasks__annotations__id', distinct=True,
                 filter=Q(tasks__annotations__was_cancelled=False)
             ),
+            num_tasks_with_annotations=Count(
+                'tasks__id', distinct=True,
+                filter=Q(tasks__annotations__isnull=False) &
+                    Q(tasks__annotations__ground_truth=False) &
+                    Q(tasks__annotations__was_cancelled=False) &
+                    Q(tasks__annotations__result__isnull=False)
+            ),
             useful_annotation_number=Count(
                 'tasks__annotations__id', distinct=True,
                 filter=Q(tasks__annotations__was_cancelled=False) &
@@ -155,7 +161,7 @@ class Project(ProjectMixin, models.Model):
 
     @property
     def num_annotations(self):
-        return Annotation.objects.filter(Q(task__project=self) & Q_finished_annotations & Q(ground_truth=False)).count()
+        return Annotation.objects.filter(task__project=self).count()
 
     @property
     def has_predictions(self):
@@ -173,10 +179,6 @@ class Project(ProjectMixin, models.Model):
     def is_private(self):
         return None
 
-    @property
-    def has_storages(self):
-        return hasattr(self, 'storages') and self.storages is not None and self.storages.count() > 0
-
     @property
     def secure_mode(self):
         return False
@@ -187,7 +189,8 @@ class Project(ProjectMixin, models.Model):
 
     @property
     def only_undefined_field(self):
-        return self.one_object_in_label_config and self.summary.common_data_columns and self.summary.common_data_columns[0] == settings.DATA_UNDEFINED_NAME
+        return self.one_object_in_label_config and self.summary.common_data_columns \
+               and self.summary.common_data_columns[0] == settings.DATA_UNDEFINED_NAME
 
     @property
     def get_labeled_count(self):
@@ -197,14 +200,6 @@ class Project(ProjectMixin, models.Model):
     def get_collected_count(self):
         return self.tasks.count()
 
-    @property
-    def num_tasks_with_annotations(self):
-        return self.tasks.filter(
-            Q(annotations__isnull=False) &
-            Q(annotations__ground_truth=False) &
-            Q_task_finished_annotations
-        ).distinct().count()
-
     @property
     def get_total_possible_count(self):
         """
@@ -277,6 +272,11 @@ class Project(ProjectMixin, models.Model):
         elif tasks_number_changed and self.overlap_cohort_percentage < 100 and self.maximum_annotations > 1:
             self._rearrange_overlap_cohort()
 
+        if maximum_annotations_changed or overlap_cohort_percentage_changed:
+            bulk_update_stats_project_tasks(self.tasks.filter(
+                Q(annotations__isnull=False) &
+                Q(annotations__ground_truth=False)))
+
     def _rearrange_overlap_cohort(self):
         tasks_with_overlap = self.tasks.filter(overlap__gt=1)
         tasks_with_overlap_count = tasks_with_overlap.count()
@@ -342,6 +342,11 @@ class Project(ProjectMixin, models.Model):
         if not hasattr(self, 'summary'):
             return
 
+        if self.num_tasks == 0:
+            logger.debug(f'Project {self} has no tasks: nothing to validate here. Ensure project summary is empty')
+            self.summary.reset()
+            return
+
         # validate data columns consistency
         fields_from_config = get_all_object_tag_names(config_string)
         if not fields_from_config:
@@ -351,7 +356,13 @@ class Project(ProjectMixin, models.Model):
         fields_from_data.discard(settings.DATA_UNDEFINED_NAME)
         if fields_from_data and not fields_from_config.issubset(fields_from_data):
             different_fields = list(fields_from_config.difference(fields_from_data))
-            raise ValidationError(f'These fields are not present in the data: {",".join(different_fields)}')
+            raise LabelStudioValidationErrorSentryIgnored(f'These fields are not present in the data: {",".join(different_fields)}')
+
+        if self.num_annotations == 0:
+            logger.debug(f'Project {self} has no annotations: nothing to validate here. '
+                         f'Ensure annotations-related project summary is empty')
+            self.summary.reset(tasks_data_based=False)
+            return
 
         # validate annotations consistency
         annotations_from_config = set(get_all_control_tag_tuples(config_string))
@@ -368,8 +379,8 @@ class Project(ProjectMixin, models.Model):
                     f'{self.summary.created_annotations[ann_tuple]} '
                     f'with from_name={from_name}, to_name={to_name}, type={t}')
             diff_str = '\n'.join(diff_str)
-            raise ValidationError(f'Created annotations are incompatible with provided labeling schema, '
-                                  f'we found:\n{diff_str}')
+            raise LabelStudioValidationErrorSentryIgnored(
+                f'Created annotations are incompatible with provided labeling schema, we found:\n{diff_str}')
 
         # validate labels consistency
         labels_from_config = get_all_labels(config_string)
@@ -377,14 +388,14 @@ class Project(ProjectMixin, models.Model):
         for control_tag_from_data, labels_from_data in created_labels.items():
             # Check if labels created in annotations, and their control tag has been removed
             if labels_from_data and control_tag_from_data not in labels_from_config:
-                raise ValidationError(
+                raise LabelStudioValidationErrorSentryIgnored(
                     f'There are {sum(labels_from_data.values(), 0)} annotation(s) created with tag '
                     f'"{control_tag_from_data}", you can\'t remove it')
             labels_from_config_by_tag = set(labels_from_config[control_tag_from_data])
             if not set(labels_from_data).issubset(set(labels_from_config_by_tag)):
                 different_labels = list(set(labels_from_data).difference(labels_from_config_by_tag))
                 diff_str = '\n'.join(f'{l} ({labels_from_data[l]} annotations)' for l in different_labels)
-                raise ValidationError(f'These labels still exist in annotations:\n{diff_str}')
+                raise LabelStudioValidationErrorSentryIgnored(f'These labels still exist in annotations:\n{diff_str}')
 
     def _label_config_has_changed(self):
         return self.label_config != self.__original_label_config
@@ -438,6 +449,13 @@ class Project(ProjectMixin, models.Model):
             self.__maximum_annotations = self.maximum_annotations
             self.__overlap_cohort_percentage = self.overlap_cohort_percentage
 
+        if hasattr(self, 'summary'):
+            # Ensure project.summary is consistent with current tasks / annotations
+            if self.num_tasks == 0:
+                self.summary.reset()
+            elif self.num_annotations == 0:
+                self.summary.reset(tasks_data_based=False)
+
     def get_member_ids(self):
         if hasattr(self, 'team_link'):
             # project has defined team scope
@@ -503,11 +521,6 @@ class Project(ProjectMixin, models.Model):
         task, _, _ = get_sample_task(config)
         return task
 
-    def pretty_model_version(self):
-        if not self.model_version:
-            return 'Undefined model version'
-        return pretty_date(self.model_version)
-
     def eta(self):
         """
             Show eta for project to be finished
@@ -643,6 +656,14 @@ class ProjectSummary(models.Model):
     def has_permission(self, user):
         return self.project.has_permission(user)
 
+    def reset(self, tasks_data_based=True):
+        if tasks_data_based:
+            self.all_data_columns = {}
+            self.common_data_columns = []
+        self.created_annotations = {}
+        self.created_labels = {}
+        self.save()
+
     def update_data_columns(self, tasks):
         common_data_columns = set()
         all_data_columns = dict(self.all_data_columns)
@@ -693,12 +714,14 @@ class ProjectSummary(models.Model):
         self.save()
 
     def _get_annotation_key(self, result):
-        result_type = result.get('type')
-        if result_type in ('relation', 'rating', 'pairwise'):
+        result_type = result.get('type', None)
+        if result_type in ('relation', 'rating', 'pairwise', None):
             return None
         if 'from_name' not in result or 'to_name' not in result:
             logger.error(
-                'Unexpected annotation.result format: "from_name" or "to_name" not found in %r' % result)
+                'Unexpected annotation.result format: "from_name" or "to_name" not found in %r', result,
+                extra={'sentry_skip': True}
+            )
             return None
         result_from_name = result['from_name']
         key = get_annotation_tuple(result_from_name, result['to_name'], result_type or '')
@@ -719,8 +742,10 @@ class ProjectSummary(models.Model):
         labels = dict(self.created_labels)
         for annotation in annotations:
             results = get_attr_or_item(annotation, 'result') or []
-            for result in results:
+            if not isinstance(results, list):
+                continue
 
+            for result in results:
                 # aggregate annotation types
                 key = self._get_annotation_key(result)
                 if not key:
@@ -746,8 +771,10 @@ class ProjectSummary(models.Model):
         labels = dict(self.created_labels)
         for annotation in annotations:
             results = get_attr_or_item(annotation, 'result') or []
-            for result in results:
+            if not isinstance(results, list):
+                continue
 
+            for result in results:
                 # reduce annotation counters
                 key = self._get_annotation_key(result)
                 if key in created_annotations:
@@ -756,7 +783,7 @@ class ProjectSummary(models.Model):
                         created_annotations.pop(key)
 
                 # reduce labels counters
-                from_name = result.get('from_name')
+                from_name = result.get('from_name', None)
                 if from_name not in labels:
                     continue
                 for label in self._get_labels(result):
