@@ -23,9 +23,6 @@ _api_urlpatterns = [
     path('', api.ProjectListAPI.as_view(), name='project-list'),
     path('<int:pk>/', api.ProjectAPI.as_view(), name='project-detail'),
 
-    # Duplicate project
-    path('<int:pk>/duplicate/', api.ProjectDuplicateAPI.as_view(), name='project-duplicate'),
-
     # Get next task
     path('<int:pk>/next/', api.ProjectNextTaskAPI.as_view(), name='project-next'),
 
@@ -43,6 +40,10 @@ _api_urlpatterns = [
 
     # Generate sample task for this project
     path('<int:pk>/sample-task/', api.ProjectSampleTask.as_view(), name='project-sample-task'),
+
+    # List available model versions
+    path('<int:pk>/model-versions/', api.ProjectModelVersions.as_view(), name='project-model-versions'),
+
 ]
 
 _api_urlpatterns_templates = [
