@@ -7,39 +7,44 @@ from django.utils import timezone
 
 import drf_yasg.openapi as openapi
 from drf_yasg.utils import swagger_auto_schema
+from django.utils.decorators import method_decorator
+from django_filters.rest_framework import DjangoFilterBackend
 
 from rest_framework.parsers import FormParser, JSONParser, MultiPartParser
-from rest_framework import generics
+from rest_framework import generics, viewsets
 from rest_framework.response import Response
-from rest_framework.permissions import IsAuthenticated
+from rest_framework.exceptions import PermissionDenied as DRFPermissionDenied
 
-from core.utils.common import get_object_with_check_and_log
-from core.decorators import permission_required
+from core.utils.common import get_object_with_check_and_log, DjangoFilterDescriptionInspector
 from core.permissions import all_permissions, ViewClassPermission
 
 from tasks.models import Task, Annotation, Prediction, AnnotationDraft
-from core.permissions import (get_object_with_permissions, check_object_permissions)
-from core.mixins import RequestDebugLogMixin
-from core.utils.common import bool_from_request
+from core.utils.common import bool_from_request, int_from_request
 from tasks.serializers import (
     TaskSerializer, AnnotationSerializer, TaskSimpleSerializer, PredictionSerializer,
-    TaskWithAnnotationsAndPredictionsAndDraftsSerializer, AnnotationDraftSerializer)
+    TaskWithAnnotationsAndPredictionsAndDraftsSerializer, AnnotationDraftSerializer, PredictionQuerySerializer)
 from projects.models import Project
 
 logger = logging.getLogger(__name__)
 
 
+@method_decorator(name='post', decorator=swagger_auto_schema(
+        tags=['Tasks'],
+        operation_summary='Create task',
+        operation_description='Create a new labeling task in Label Studio.',
+        request_body=TaskSerializer))
 class TaskListAPI(generics.ListCreateAPIView):
-    """
-    post:
-    Create task
-
-    Create a new labeling task in Label Studio.
-    """
     parser_classes = (JSONParser, FormParser, MultiPartParser)
     queryset = Task.objects.all()
+    permission_required = ViewClassPermission(
+        GET=all_permissions.tasks_view,
+        POST=all_permissions.tasks_create,
+    )
     serializer_class = TaskSerializer
 
+    def filter_queryset(self, queryset):
+        return queryset.filter(project__organization=self.request.user.active_organization)
+
     @swagger_auto_schema(auto_schema=None)
     def get(self, request, *args, **kwargs):
         return super(TaskListAPI, self).get(request, *args, **kwargs)
@@ -51,30 +56,42 @@ class TaskListAPI(generics.ListCreateAPIView):
             context['project'] = generics.get_object_or_404(Project, pk=project_id)
         return context
 
-    @swagger_auto_schema(tags=['Tasks'], request_body=TaskSerializer)
     def post(self, request, *args, **kwargs):
         return super(TaskListAPI, self).post(request, *args, **kwargs)
 
 
+@method_decorator(name='get', decorator=swagger_auto_schema(
+        tags=['Tasks'],
+        operation_summary='Get task',
+        operation_description="""
+        Get task data, metadata, annotations and other attributes for a specific labeling task by task ID.
+        """,
+        manual_parameters=[
+            openapi.Parameter(name='proxy', type=openapi.TYPE_BOOLEAN, in_=openapi.IN_QUERY,
+                          description='Use the proxy parameter inline for credential access to task data')
+        ]))
+@method_decorator(name='patch', decorator=swagger_auto_schema(
+        tags=['Tasks'],
+        operation_summary='Update task',
+        operation_description='Update the attributes of an existing labeling task.',
+        request_body=TaskSimpleSerializer))
+@method_decorator(name='delete', decorator=swagger_auto_schema(
+        tags=['Tasks'],
+        operation_summary='Delete task',
+        operation_description='Delete a task in Label Studio. This action cannot be undone!',
+        ))
 class TaskAPI(generics.RetrieveUpdateDestroyAPIView):
-    """
-    get:
-    Get task by ID
-
-    Get task data, metadata, annotations and other attributes for a specific labeling task.
-
-    patch:
-    Update task
-
-    Update the attributes of an existing labeling task.
-
-    delete:
-    Delete task
-
-    Delete a task in Label Studio. This action cannot be undone!
-    """
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    queryset = Task.objects.all()
+    permission_required = ViewClassPermission(
+        GET=all_permissions.tasks_view,
+        PUT=all_permissions.tasks_change,
+        PATCH=all_permissions.tasks_change,
+        DELETE=all_permissions.tasks_delete,
+    )
+
+    def get_queryset(self):
+        return Task.objects.filter(project__organization=self.request.user.active_organization)
+
 
     def get_serializer_class(self):
         # GET => task + annotations + predictions + drafts
@@ -99,19 +116,13 @@ class TaskAPI(generics.RetrieveUpdateDestroyAPIView):
         proxy = bool_from_request(request.GET, 'proxy', True)
         result['data'] = task.resolve_uri(result['data'], proxy=proxy)
         return Response(result)
-    
-    @swagger_auto_schema(tags=['Tasks'], manual_parameters=[
-            openapi.Parameter(name='proxy', type=openapi.TYPE_BOOLEAN, in_=openapi.IN_QUERY,
-                              description='Use the proxy parameter inline for credential access to task data')
-    ])
+
     def get(self, request, *args, **kwargs):
         return super(TaskAPI, self).get(request, *args, **kwargs)
 
-    @swagger_auto_schema(tags=['Tasks'], request_body=TaskSimpleSerializer)
     def patch(self, request, *args, **kwargs):
         return super(TaskAPI, self).patch(request, *args, **kwargs)
 
-    @swagger_auto_schema(tags=['Tasks'])
     def delete(self, request, *args, **kwargs):
         return super(TaskAPI, self).delete(request, *args, **kwargs)
 
@@ -120,23 +131,22 @@ class TaskAPI(generics.RetrieveUpdateDestroyAPIView):
         return super(TaskAPI, self).put(request, *args, **kwargs)
 
 
-class AnnotationAPI(RequestDebugLogMixin, generics.RetrieveUpdateDestroyAPIView):
-    """
-    get:
-    Get annotation by its ID
-
-    Retrieve a specific annotation for a task.
-
-    patch:
-    Update annotation
-
-    Update existing attributes on an annotation. 
-
-    delete:
-    Delete annotation
-
-    Delete an annotation. This action can't be undone! 
-    """
+@method_decorator(name='get', decorator=swagger_auto_schema(
+        tags=['Annotations'],
+        operation_summary='Get annotation by its ID',
+        operation_description='Retrieve a specific annotation for a task using the annotation result ID.',
+        ))
+@method_decorator(name='patch', decorator=swagger_auto_schema(
+        tags=['Annotations'],
+        operation_summary='Update annotation',
+        operation_description='Update existing attributes on an annotation.',
+        request_body=AnnotationSerializer))
+@method_decorator(name='delete', decorator=swagger_auto_schema(
+        tags=['Annotations'],
+        operation_summary='Delete annotation',
+        operation_description='Delete an annotation. This action can\'t be undone!',
+        ))
+class AnnotationAPI(generics.RetrieveUpdateDestroyAPIView):
     parser_classes = (JSONParser, FormParser, MultiPartParser)
     permission_required = ViewClassPermission(
         GET=all_permissions.annotations_view,
@@ -158,37 +168,52 @@ class AnnotationAPI(RequestDebugLogMixin, generics.RetrieveUpdateDestroyAPIView)
 
         annotation.task.save()  # refresh task metrics
 
+        if self.request.data.get('ground_truth'):
+            annotation.task.ensure_unique_groundtruth(annotation_id=annotation.id)
+
         return super(AnnotationAPI, self).update(request, *args, **kwargs)
 
-    @swagger_auto_schema(tags=['Annotations'])
     def get(self, request, *args, **kwargs):
         return super(AnnotationAPI, self).get(request, *args, **kwargs)
 
-    @swagger_auto_schema(tags=['Annotations'], auto_schema=None)
+    @swagger_auto_schema(auto_schema=None)
     def put(self, request, *args, **kwargs):
         return super(AnnotationAPI, self).put(request, *args, **kwargs)
 
-    @swagger_auto_schema(tags=['Annotations'], request_body=AnnotationSerializer)
     def patch(self, request, *args, **kwargs):
         return super(AnnotationAPI, self).patch(request, *args, **kwargs)
 
-    @swagger_auto_schema(tags=['Annotations'])
     def delete(self, request, *args, **kwargs):
         return super(AnnotationAPI, self).delete(request, *args, **kwargs)
 
+
+@method_decorator(name='get', decorator=swagger_auto_schema(
+        tags=['Annotations'],
+        operation_summary='Get all task annotations',
+        operation_description='List all annotations for a task.',
+        ))
+@method_decorator(name='post', decorator=swagger_auto_schema(
+        tags=['Annotations'],
+        operation_summary='Create annotation',
+        operation_description="""
+        Add annotations to a task like an annotator does. The content of the result field depends on your 
+        labeling configuration. For example, send the following data as part of your POST 
+        request to send an empty annotation with the ID of the user who completed the task:
         
-class AnnotationsListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
-    """
-    get:
-    Get all task annotations
-
-    List all annotations for a task.
-
-    post:
-    Create new annotation
-
-    Add annotations to a task like an annotator does.
-    """
+        ```json
+        {
+        "result": {},
+        "was_cancelled": true,
+        "ground_truth": true,
+        "lead_time": 0,
+        "task": 0
+        "completed_by": {"id": 1, "email": "heartex@example.com", "first_name": "", "last_name": ""}
+        } 
+        ```
+        """,
+        request_body=AnnotationSerializer
+        ))
+class AnnotationsListAPI(generics.ListCreateAPIView):
     parser_classes = (JSONParser, FormParser, MultiPartParser)
     permission_required = ViewClassPermission(
         GET=all_permissions.annotations_view,
@@ -197,11 +222,9 @@ class AnnotationsListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
 
     serializer_class = AnnotationSerializer
 
-    @swagger_auto_schema(tags=['Annotations'])
     def get(self, request, *args, **kwargs):
         return super(AnnotationsListAPI, self).get(request, *args, **kwargs)
 
-    @swagger_auto_schema(tags=['Annotations'], request_body=AnnotationSerializer)
     def post(self, request, *args, **kwargs):
         return super(AnnotationsListAPI, self).post(request, *args, **kwargs)
 
@@ -213,9 +236,6 @@ class AnnotationsListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
         task = get_object_with_check_and_log(self.request, Task, pk=self.kwargs['pk'])
         # annotator has write access only to annotations and it can't be checked it after serializer.save()
         user = self.request.user
-        # Release task if it has been taken at work (it should be taken by the same user, or it makes sentry error
-        logger.debug(f'User={user} releases task={task}')
-        task.release_lock(user)
 
         # updates history
         update_id = self.request.user.id
@@ -250,16 +270,23 @@ class AnnotationsListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
         self.request.user.activity_at = timezone.now()
         self.request.user.save()
 
+        # Release task if it has been taken at work (it should be taken by the same user, or it makes sentry error
+        logger.debug(f'User={user} releases task={task}')
+        task.release_lock(user)
+
         # if annotation created from draft - remove this draft
         draft_id = self.request.data.get('draft_id')
         if draft_id is not None:
             logger.debug(f'Remove draft {draft_id} after creating annotation {annotation.id}')
             AnnotationDraft.objects.filter(id=draft_id).delete()
 
+        if self.request.data.get('ground_truth'):
+            annotation.task.ensure_unique_groundtruth(annotation_id=annotation.id)
+
         return annotation
 
 
-class AnnotationDraftListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
+class AnnotationDraftListAPI(generics.ListCreateAPIView):
 
     parser_classes = (JSONParser, MultiPartParser, FormParser)
     serializer_class = AnnotationDraftSerializer
@@ -270,6 +297,10 @@ class AnnotationDraftListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
     queryset = AnnotationDraft.objects.all()
     swagger_schema = None
 
+    def filter_queryset(self, queryset):
+        task_id = self.kwargs['pk']
+        return queryset.filter(task_id=task_id)
+
     def perform_create(self, serializer):
         task_id = self.kwargs['pk']
         annotation_id = self.kwargs.get('annotation_id')
@@ -282,7 +313,7 @@ class AnnotationDraftListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
         )
 
 
-class AnnotationDraftAPI(RequestDebugLogMixin, generics.RetrieveUpdateDestroyAPIView):
+class AnnotationDraftAPI(generics.RetrieveUpdateDestroyAPIView):
 
     parser_classes = (JSONParser, MultiPartParser, FormParser)
     serializer_class = AnnotationDraftSerializer
@@ -294,3 +325,37 @@ class AnnotationDraftAPI(RequestDebugLogMixin, generics.RetrieveUpdateDestroyAPI
         DELETE=all_permissions.annotations_delete,
     )
     swagger_schema = None
+
+
+@method_decorator(name='list', decorator=swagger_auto_schema(
+    tags=['Predictions'], operation_summary="List predictions", 
+    filter_inspectors=[DjangoFilterDescriptionInspector],
+    operation_description="List all predictions."))
+@method_decorator(name='create', decorator=swagger_auto_schema(
+    tags=['Predictions'],
+    operation_summary="Create prediction",
+    operation_description="Create a prediction."))
+@method_decorator(name='retrieve', decorator=swagger_auto_schema(
+    tags=['Predictions'],
+    operation_summary="Get prediction",
+    operation_description="Get all predictions in an organization, or for a specific task or project by ID."))
+@method_decorator(name='update', decorator=swagger_auto_schema(
+    tags=['Predictions'],
+    operation_summary="Put prediction",
+    operation_description="Overwrite prediction data by prediction ID."))
+@method_decorator(name='partial_update', decorator=swagger_auto_schema(
+    tags=['Predictions'],
+    operation_summary="Update prediction",
+    operation_description="Update prediction data by prediction ID."))
+@method_decorator(name='destroy', decorator=swagger_auto_schema(
+    tags=['Predictions'],
+    operation_summary="Delete prediction",
+    operation_description="Delete a prediction by prediction ID."))
+class PredictionAPI(viewsets.ModelViewSet):
+    serializer_class = PredictionSerializer
+    permission_required = all_permissions.predictions_any
+    filter_backends = [DjangoFilterBackend]
+    filterset_fields = ['task', 'task__project']
+
+    def get_queryset(self):
+        return Prediction.objects.filter(task__project__organization=self.request.user.active_organization)
