@@ -23,6 +23,11 @@ from users.serializers import UserSerializer
 logger = logging.getLogger(__name__)
 
 
+class PredictionQuerySerializer(serializers.Serializer):
+    task = serializers.IntegerField(required=False, help_text='Task ID to filter predictions')
+    task__project = serializers.IntegerField(required=False, help_text='Project ID to filter predictions')
+
+
 class PredictionSerializer(ModelSerializer):
     model_version = serializers.CharField(allow_blank=True)
     created_ago = serializers.CharField(default='', read_only=True, help_text='Delta time from creation time')
@@ -39,9 +44,10 @@ class ListAnnotationSerializer(serializers.ListSerializer):
 class AnnotationSerializer(DynamicFieldsMixin, ModelSerializer):
     """
     """
-    created_username = serializers.SerializerMethodField(default='', read_only=True, help_text='User name string')
-    created_ago = serializers.CharField(default='', read_only=True, help_text='Delta time from creation time')
-    completed_by = serializers.SerializerMethodField()
+    created_username = serializers.SerializerMethodField(default='', read_only=True, help_text='Username string')
+    created_ago = serializers.CharField(default='', read_only=True, help_text='Time delta from creation time')
+    completed_by = serializers.SerializerMethodField(default='Requesting user',
+                                                     help_text='ID of the user who completed the annotation.')
 
     @classmethod
     def many_init(cls, *args, **kwargs):
@@ -73,7 +79,7 @@ class AnnotationSerializer(DynamicFieldsMixin, ModelSerializer):
         if not isinstance(data, list):
             raise ValidationError('annotation "result" field in annotation must be list')
 
-        return value
+        return data
 
     def get_created_username(self, annotation):
         user = annotation.completed_by
@@ -212,17 +218,34 @@ class TaskSerializerBulk(serializers.ListSerializer):
 
         return ret
 
-    @staticmethod
-    def get_completed_by_id(annotation, default=None):
-        completed_by = annotation.get('completed_by', None)
-        # user id as is
-        if completed_by and isinstance(completed_by, int):
-            return completed_by
-        # user dict
-        if completed_by and isinstance(completed_by, dict):
-            return completed_by.get('id')
+    def _insert_valid_completed_by_id_or_raise(self, annotations, members_email_to_id, members_ids, default_user):
+        for annotation in annotations:
+            completed_by = annotation.get('completed_by')
+            # no completed_by info found - just skip it, will be assigned to the user who imports
+            if completed_by is None:
+                annotation['completed_by_id'] = default_user.id
 
-        return default
+            # resolve annotators by email
+            elif isinstance(completed_by, dict):
+                if 'email' not in completed_by:
+                    raise ValidationError(f"It's expected to have 'email' field in 'completed_by' data in annotations")
+                email = completed_by['email']
+                if email not in members_email_to_id:
+                    raise ValidationError(f"Unknown annotator's email {email}")
+                # overwrite an actual member ID
+                annotation['completed_by_id'] = members_email_to_id[email]
+
+            # old style annotators specification - try to find them by ID
+            elif isinstance(completed_by, int) and completed_by in members_ids:
+                if completed_by not in members_ids:
+                    raise ValidationError(f"Unknown annotator's ID {completed_by}")
+                annotation['completed_by_id'] = completed_by
+
+            # in any other cases - import validation error
+            else:
+                raise ValidationError(
+                    f"Import data contains completed_by={completed_by} which is not a valid annotator's email or ID")
+            annotation.pop('completed_by', None)
 
     @retry_database_locked()
     def create(self, validated_data):
@@ -233,28 +256,25 @@ class TaskSerializerBulk(serializers.ListSerializer):
         user = self.context.get('user', None)
         project = self.context.get('project')
 
+        organization = user.active_organization \
+            if not project.created_by.active_organization else project.created_by.active_organization
+        members_email_to_id = dict(organization.members.values_list('user__email', 'user__id'))
+        members_ids = set(members_email_to_id.values())
+        logger.debug(f"{len(members_email_to_id)} members found in organization {organization}")
+
         # to be sure we add tasks with annotations at the same time
         with transaction.atomic():
 
             # extract annotations and predictions
             task_annotations, task_predictions = [], []
             for task in validated_tasks:
-                task_annotations.append(task.pop('annotations', []))
-                task_predictions.append(task.pop('predictions', []))
-
-            # check annotator permissions for completed by
-            organization = user.active_organization \
-                if not project.created_by.active_organization else project.created_by.active_organization
-            project_user_ids = organization.members.values_list('user__id', flat=True)
-            annotator_ids = set()
-            for annotations in task_annotations:
-                for annotation in annotations:
-                    annotator_ids.add(self.get_completed_by_id(annotation))
-
-            for i in annotator_ids:
-                if i not in project_user_ids and i is not None:
-                    raise ValidationError(f'Annotations with "completed_by"={i} are produced by annotator '
-                                          f'who is not allowed for this project as invited annotator or team member')
+                annotations = task.pop('annotations', [])
+                # insert a valid "completed_by_id" by existing member
+                self._insert_valid_completed_by_id_or_raise(
+                    annotations, members_email_to_id, members_ids, user or project.created_by)
+                predictions = task.pop('predictions', [])
+                task_annotations.append(annotations)
+                task_predictions.append(predictions)
 
             # add tasks first
             for task in validated_tasks:
@@ -284,16 +304,12 @@ class TaskSerializerBulk(serializers.ListSerializer):
                 for annotation in annotations:
                     # support both "ground_truth" and "ground_truth"
                     ground_truth = annotation.pop('ground_truth', True)
-                    if 'ground_truth' in annotation:
-                        ground_truth = annotation.pop('ground_truth', True)
-
-                    # get user id
-                    completed_by_id = self.get_completed_by_id(annotation, default=user.id if user else None)
-                    annotation.pop('completed_by', None)
+                    was_cancelled = annotation.pop('was_cancelled', False)
 
                     db_annotations.append(Annotation(task=self.db_tasks[i],
                                                      ground_truth=ground_truth,
-                                                     completed_by_id=completed_by_id,
+                                                     was_cancelled=was_cancelled,
+                                                     completed_by_id=annotation['completed_by_id'],
                                                      result=annotation['result']))
 
             # add predictions
@@ -423,12 +439,14 @@ class AnnotationDraftSerializer(ModelSerializer):
 
     def get_created_username(self, draft):
         user = draft.user
+        if not user:
+            return ""
 
         name = user.first_name
         if len(user.last_name):
             name = name + " " + user.last_name
 
-        name += f' ({user.email}, {user.id})'
+        name += (' ' if name else '') + f'{user.email}, {user.id}'
         return name
 
     class Meta:
@@ -438,10 +456,16 @@ class AnnotationDraftSerializer(ModelSerializer):
 
 class TaskWithAnnotationsAndPredictionsAndDraftsSerializer(TaskSerializer):
 
-    predictions = PredictionSerializer(many=True, default=[], read_only=True)
+    predictions = serializers.SerializerMethodField(default=[], read_only=True)
     annotations = serializers.SerializerMethodField(default=[], read_only=True)
     drafts = serializers.SerializerMethodField(default=[], read_only=True)
 
+    def get_predictions(self, task):
+        predictions = task.predictions
+        if task.project.model_version:
+            predictions = predictions.filter(model_version=task.project.model_version)
+        return PredictionSerializer(predictions, many=True, read_only=True, default=[], context=self.context).data
+
     def get_annotations(self, task):
         """Return annotations only for the current user"""
         annotations = task.annotations
@@ -451,7 +475,7 @@ class TaskWithAnnotationsAndPredictionsAndDraftsSerializer(TaskSerializer):
             if user.is_annotator:
                 annotations = annotations.filter(completed_by=user)
 
-        return AnnotationSerializer(annotations, many=True, read_only=True, default=True, context=self.context).data
+        return AnnotationSerializer(annotations, many=True, read_only=True, default=[], context=self.context).data
 
     def get_drafts(self, task):
         """Return drafts only for the current user"""
@@ -462,11 +486,9 @@ class TaskWithAnnotationsAndPredictionsAndDraftsSerializer(TaskSerializer):
         drafts = task.drafts
         if 'request' in self.context and hasattr(self.context['request'], 'user'):
             user = self.context['request'].user
-            # drafts = drafts.filter(user=user)
-            if user.is_annotator:
-                drafts = drafts.filter(user=user)
+            drafts = drafts.filter(user=user)
 
-        return AnnotationDraftSerializer(drafts, many=True, read_only=True, default=True, context=self.context).data
+        return AnnotationDraftSerializer(drafts, many=True, read_only=True, default=[], context=self.context).data
 
 
 class TaskIDWithAnnotationsAndPredictionsSerializer(ModelSerializer):
