@@ -13,20 +13,38 @@ from copy import deepcopy
 from django.conf import settings
 from projects.models import Project
 from tasks.models import Task
-from tests.utils import make_project
 from users.models import User
 from organizations.models import Organization
 from types import SimpleNamespace
 
+# if we haven't this package, pytest.ini::env doesn't work
+try:
+    import pytest_env.plugin
+except ImportError:
+    print('\n\n !!! Please, pip install pytest-env \n\n')
+    exit(-100)
+
 from .utils import (
     create_business, signin, gcs_client_mock, ml_backend_mock, register_ml_backend_mock, azure_client_mock,
-    redis_client_mock
+    redis_client_mock, make_project
 )
 
-
 boto3.set_stream_logger('botocore.credentials', logging.DEBUG)
 
 
+@pytest.fixture(autouse=True)
+def disable_sentry():
+    settings.SENTRY_RATE = 0
+    settings.SENTRY_DSN = None
+
+
+@pytest.fixture(scope="function")
+def enable_sentry():
+    settings.SENTRY_RATE = 0
+    # it's disabled key, but this is correct
+    settings.SENTRY_DSN = 'https://44f7a50de5ab425ca6bc406ef69b2122@o227124.ingest.sentry.io/5820521'
+
+
 @pytest.fixture(scope='function')
 def aws_credentials():
     """Mocked AWS Credentials for moto."""
@@ -62,23 +80,28 @@ def s3_with_images(s3):
     s3.put_object(Bucket=bucket_name, Key='image1.jpg', Body='123')
     s3.put_object(Bucket=bucket_name, Key='subdir/image1.jpg', Body='456')
     s3.put_object(Bucket=bucket_name, Key='subdir/image2.jpg', Body='789')
+    s3.put_object(Bucket=bucket_name, Key='subdir/another/image2.jpg', Body='0ab')
     yield s3
 
 
 @pytest.fixture(autouse=True)
 def s3_with_jsons(s3):
-    """
-    Bucket structure:
-    s3://pytest-s3-images/image1.jpg
-    s3://pytest-s3-images/subdir/image1.jpg
-    s3://pytest-s3-images/subdir/image2.jpg
-    """
     bucket_name = 'pytest-s3-jsons'
     s3.create_bucket(Bucket=bucket_name)
     s3.put_object(Bucket=bucket_name, Key='test.json', Body=json.dumps({'image_url': 'http://ggg.com/image.jpg'}))
     yield s3
 
 
+@pytest.fixture(autouse=True)
+def s3_with_hypertext_s3_links(s3):
+    bucket_name = 'pytest-s3-jsons-hypertext'
+    s3.create_bucket(Bucket=bucket_name)
+    s3.put_object(Bucket=bucket_name, Key='test.json', Body=json.dumps({
+        'text': "<a href=\"s3://hypertext-bucket/file with /spaces and' / ' / quotes.jpg\"/>"
+    }))
+    yield s3
+
+
 @pytest.fixture(autouse=True)
 def s3_export_bucket(s3):
     bucket_name = 'pytest-export-s3-bucket'
@@ -270,7 +293,8 @@ def business_client(client):
     client.user = user
     client.organization = org
 
-    assert signin(client, email, password).status_code == 302
+    if signin(client, email, password).status_code != 302:
+        print(f'User {user} failed to login!')
     return client
 
 
@@ -285,7 +309,8 @@ def annotator_client(client):
     user.save()
     business = create_business(user)
     Organization.create_organization(created_by=user, title=user.first_name)
-    assert signin(client, email, password).status_code == 302
+    if signin(client, email, password).status_code != 302:
+        print(f'User {user} failed to login!')
     client.user = user
     client.annotator = user
     return client
@@ -302,7 +327,8 @@ def annotator2_client(client):
     user.save()
     business = create_business(user)
     Organization.create_organization(created_by=user, title=user.first_name)
-    assert signin(client, email, password).status_code == 302
+    if signin(client, email, password).status_code != 302:
+        print(f'User {user} failed to login!')
     client.user = user
     client.annotator = user
     return client
@@ -343,3 +369,8 @@ def configured_project(business_client, annotator_client):
 
     Task.objects.bulk_create([Task(data=task, project=project) for task in _2_tasks_with_textA_and_textB])
     return project
+
+
+@pytest.fixture(name="django_live_url")
+def get_server_url(live_server):
+    yield live_server.url
